%Sequalizer V1.0
%Authors: Fahim Farzadfard & Nava Gharaei
%This script uses a modified version of SeqDoc (Crowe, M. L.
%(2005). BMC Bioinformatics, 6(1), 133). The path to this script should be 
%specifid in the input parameters.
% Perl and  SeqDoc dependencies need to be installed.


%input parameters
% 1. Specify path to the modified SeqDoc script 
SeqDoc_path='C:\Sequalizer_SeqDoc.pl';
% 2. Select the reference Sanger chromatogram. The reference 
%chromatogram will be compared with all the other Sanger chromatograms in the
%same directory (i.e., test samples). The output of Sequalizer will be saved in the
%same directory.
[FileName,PathName] = uigetfile('*.ab1','Select the reference file within directory that contains test files');
% 3. Specify sequence of the WT taget. It is used to align Sanger chromatograms to the target.
WT='CCCAATGGCGATTAAATGGCCTAAATAGAGCCTATGGG';
% 4. Specify positions of the targeted dC or dG residues 
target=[18:21];
% end of input parameters

%Analyze the reference chromatogram by SeqDoc
fileID = fopen([PathName 'input_parameters.txt'],'w');
fprintf(fileID,'%s\n',['WT=' WT char(9) num2str(target)]);
fclose(fileID);
y1=-5000;y2=5000;
fileID = fopen([PathName 'output.txt'],'w');
tm=0;
ref=[PathName FileName];
system([SeqDoc_path ' ' ref ' ' ref ' ' PathName]);
imp_ref=textread([PathName 'ref.txt'],'%s','delimiter','\n');
t = ~cellfun(@isempty, regexp(imp_ref,'=>'));
a=find(t==1);
labels_ref=strrep(split(imp_ref(a),' =>'),'''','');
st='';
st=['''' char(labels_ref(1,1)) ''',[]'];
val=[];
for i=2:size(labels_ref,1);
    st=[st ',''' char(labels_ref(i,1)) ''',[]'];
end
REF=eval(['struct(' st ')']);
id_seq=find(ismember(labels_ref, 'sequence')); REF.sequence=char(replace(labels_ref(id_seq,2),{' ',',',''''},''));
lab={'a_trace','c_trace','g_trace','t_trace','base_pos'};
for u=1:size(lab,2)
id_pos(u)=find(ismember(labels_ref, lab(u)));
if id_pos(u)<size(labels_ref,1)
O=str2num(char(replace(imp_ref(a(id_pos(u))+1:a(id_pos(u)+1)-2),',','')));
else
O=str2num(char(replace(imp_ref(a(id_pos(u))+1:end-2),',','')));
end
if u==1
    REF.a_trace=O;
elseif u==2
    REF.c_trace=O;
elseif u==3
    REF.g_trace=O;
elseif u==4
    REF.t_trace=O;
elseif u==5
    REF.base_pos=O;
end
   O='' ;
end
xp=localalign(WT,REF.sequence);
start=REF.base_pos(xp.Start(2));stop=REF.base_pos(xp.Stop(2)-tm);
k=0;
for k=0:xp.Stop(2)-tm-xp.Start(2)
    u=REF.sequence(xp.Start(2)+k);
    lb{k+1}=u;
end
tout_labels='filename';
for kk=1:size(target,2)
    tout_labels=[tout_labels char(9) char(lb(target(kk))) num2str(target(kk))];
end
fprintf(fileID,'%s\n',tout_labels);
%End of reference chromatogram analysis


%Analyze each test chromatogram and compare with reference
%estimate position-specific mutation frequencies in the targeted positions
%Plot the differences between normalized chromaogram of ref and test
%samples and export position specific mutation frequencies to an output file.
files=dir([PathName '*.ab1']);
for z=1:size(files,1)
filename=[PathName files(z).name];
system([SeqDoc_path ' ' ref ' ' filename ' ' PathName]);
imp=textread([PathName 'diff.txt'],'%s','delimiter','\n');
imp=replace(imp,'''','');imp=replace(imp,',','');
labels={'a_trace => [', 'c_trace => [', 'g_trace => [', 't_trace => ['};
ids={'A', 'C', 'G', 'T'};
for i=1:size(labels,2)
ind(i)= find(ismember(imp, labels(i)));
end
[ord d]=sort(ind);
leng=ord(2)-ord(1)-2;
diff=struct(ids{d(1)},str2num(char(imp(ord(1)+1:ord(1)+leng))),ids{d(2)},str2num(char(imp(ord(2)+1:ord(2)+leng))),ids{d(3)},str2num(char(imp(ord(3)+1:ord(3)+leng))),ids{d(4)},str2num(char(imp(ord(4)+1:ord(4)+leng))));
pl=traceplot(diff);
pl.A.LineWidth=2;pl.C.LineWidth=2;pl.G.LineWidth=2;pl.T.LineWidth=2;pl.C.Color=[0 0 0];pl.G.Color=[0.9 .65 1];
xlim([start stop]);
ylim([y1 y2]);
title(strrep(files(z).name,'_','\_'));
xticks(REF.base_pos(xp.Start(2):xp.Stop(2)-tm));
xticklabels(lb);
set(gca, 'fontsize', 10, 'fontweight', 'b');
for p=1:size(target,2)
    if WT(target(p))=='G'
        xl=REF.base_pos(xp.Start(2)+target(p)-1);
        yl=diff.G(REF.base_pos(xp.Start(2)+target(p)-1));
    val(p)=round(real(sqrt((max(diff.G(REF.base_pos(xp.Start(2)+target(p)-1)-2:REF.base_pos(xp.Start(2)+target(p)-1)+2))-min(diff.A(REF.base_pos(xp.Start(2)+target(p)-1)-2:REF.base_pos(xp.Start(2)+target(p)-1)+2)))/(8*max(REF.g_trace(REF.base_pos(xp.Start(2)+target(p)-1)-2:REF.base_pos(xp.Start(2)+target(p)-1)+2))))),3);
    elseif WT(target(p))=='C'
        xl=REF.base_pos(xp.Start(2)+target(p)-1);
        yl=diff.C(REF.base_pos(xp.Start(2)+target(p)-1));
    val(p)=round(real(sqrt((max(diff.C(REF.base_pos(xp.Start(2)+target(p)-1)-2:REF.base_pos(xp.Start(2)+target(p)-1)+2))-min(diff.T(REF.base_pos(xp.Start(2)+target(p)-1)-2:REF.base_pos(xp.Start(2)+target(p)-1)+2)))/(8*max(REF.c_trace(REF.base_pos(xp.Start(2)+target(p)-1)-2:REF.base_pos(xp.Start(2)+target(p)-1)+2))))),3);
    else
        val(p)=0;
    end
    h=text(xl,yl+200,num2str(val(p)));
    set(h, 'rotation', 90)
end
fprintf(fileID,'%s\n',[files(z).name char(9) replace(regexprep(num2str(val),' +',' '),' ',char(9))]);
saveas(gcf,[PathName files(z).name '.jpg']);
end
fclose(fileID);

